/* Udvuleanu Adrian 332CB*/
#include "usart.h"
#include <avr/io.h>
#include <util/delay.h>
#include <avr/interrupt.h>
#include <stdlib.h>
#include <string.h>

#define RESET_MOTOR 0x7A1
#define TIMER_FREQ 0x10
#define TIMER_ENGINE_STOP 0xFF
#define SPEED1 0.4
#define SPEED2 0.2
#define SPEED3 0

volatile char command;
volatile char too_close = 0;
volatile char rsignal=0, lsignal=0;
volatile int timer;
volatile int ledd7;
unsigned int rx_len;
char rx_buf[18];

ISR(TIMER0_OVF_vect) {
	PORTB &= ~(1 << PB4);
	PORTB &= ~(1 << PB5);
}

ISR(TIMER0_COMPB_vect) {
	PORTB |= (1 << PB4);
	PORTB |= (1 << PB5);
}

// ISR(TIMER1_OVF_vect) {
//     if(ledd7)
// 	    PORTD ^= (1 << PD7);
// 	else PORTD &= ~(1 << PD7);
// }

ISR(PCINT2_vect) {
	if ((PINC & (1 << PC5)) == 1) {					// Checks if echo is high
		TCNT1 = 0;								// Reset Timer
		PORTD |= (1 << PD7);  //debugging led 
	} else {
		uint16_t numuS = TCNT1;					// Save Timer value
		uint8_t oldSREG = SREG;
		if(numuS/58 < 100)
			too_close = 1;
		else
			too_close = 0;
		SREG = oldSREG;							// Enable interrupts
		//PORTC &= ~(1 << PD7);						// Toggle debugging LED
	}
}

ISR(USART0_RX_vect) {
	char ReceivedByte;
	ReceivedByte = UDR0;
	rx_buf[rx_len++] = ReceivedByte;
	if(ReceivedByte == '\r') {
        command = rx_buf[rx_len - 2];
		rx_len = 0;
		timer = 2;
	}
	else if(rx_len == 16)
		rx_len = 0; 
}


void activate_motor() {
	if (too_close == 1)
		command='0';
    switch(command) {
		case '1':
			//backwards left
			PORTB |= (1 << PB2);
			PORTB &= ~(1 << PB3);
			PORTB |= (1 << PB1);
			PORTB &= ~(1 << PB0);
			OCR0B = TIMER_FREQ * SPEED3;
			break;
		case '2':
			//forward slow
			PORTB |= (1 << PB0);
			PORTB &= ~(1 << PB1);
			PORTB |= (1 << PB2);
			PORTB &= ~(1 << PB3);
			OCR0B = TIMER_FREQ * SPEED2;
			break;
		case '3':
			//forward fast
			PORTB |= (1 << PB0);
			PORTB &= ~(1 << PB1);
			PORTB |= (1 << PB2);
			PORTB &= ~(1 << PB3);
			OCR0B = TIMER_FREQ * SPEED3;
			break;
		case '4':
			//backwards right
			PORTB |= (1 << PB0);
			PORTB &= ~(1 << PB1);
			PORTB |= (1 << PB3);
			PORTB &= ~(1 << PB2);
			OCR0B = TIMER_FREQ * SPEED3;
			break;
		case '5':
			//backwards slowly
			PORTB |= (1 << PB1);
			PORTB &= ~(1 << PB0);
			PORTB |= (1 << PB3);
			PORTB &= ~(1 << PB2);
			OCR0B = TIMER_FREQ * SPEED2;
			break;
		case '6':
			//inapoi repede
			PORTB |= (1 << PB1);
			PORTB &= ~(1 << PB0);
			PORTB |= (1 << PB3);
			PORTB &= ~(1 << PB2);
			OCR0B = TIMER_FREQ * SPEED3;
			break;
		case '0':
		    OCR0B = TIMER_ENGINE_STOP;
			break;
	    case '8':
			//forward left
	        PORTB |= (1 << PB0);
			PORTB &= ~(1 << PB1);
			PORTB |= (1 << PB3);
			PORTB &= ~(1 << PB2);
			OCR0B = TIMER_FREQ * SPEED3;
	        break;
	    case '9':
			//forward right
	        PORTB |= (1 << PB2);
			PORTB &= ~(1 << PB3);
			PORTB |= (1 << PB1);
			PORTB &= ~(1 << PB0);
			OCR0B = TIMER_FREQ * SPEED3;
	        break;
	    case '7':
			OCR0B = TIMER_ENGINE_STOP;
	        break;
	    case 'h':
	        lsignal = 1;
			rsignal = 1;
	        break;
		case 'r':
	        lsignal = 1;
	        break;
		case 't':
	        rsignal = 1;
	        break;
		case 'y':
	        rsignal = 0;
			lsignal = 0;
	        break;
		default:
			break;

	}

}

void init_engine() {
	//set pins as output
	DDRB |= (1 << PB0) | (1 << PB1) | (1 << PB2) | (1 << PB3) | (1 << PB4) | (1 << PB5);
	//set pin value low
	PORTB = 0;

	//timer 0 fast pwm prescaler 64
	TCCR0A |= (1 << WGM01) | (1 << WGM00);
	TCCR0B |= (1 << WGM02);
	TCCR0B |= (1 << CS02) | (1 << CS00);
	TIMSK0 |= (1 << OCIE0B) | (1 << TOIE0);
	
	OCR0A = TIMER_FREQ;
	OCR0B = TIMER_ENGINE_STOP;
	
}

ISR(TIMER1_OVF_vect) {
	if (lsignal)
		PORTA ^= (1 << PA0);
	else
		PORTA &= ~(1 << PA0);
	if (rsignal)
		PORTA ^= (1 << PA1);
	else
		PORTA &= ~(1 << PA1);
}

ISR(TIMER1_COMPB_vect) {
	if (lsignal)
		PORTA ^= (1 << PA0);
	else
		PORTA &= ~(1 << PA0);
	if (rsignal)
		PORTA ^= (1 << PA1);
	else
		PORTA &= ~(1 << PA1);
}

void init_led() {
    DDRD |= (1 << PD7);
    PORTD &= ~(1 << PD7);

	DDRA |= (1 << PA0);
	DDRA |= (1 << PA1);
	PORTA &= ~(1 << PA0);
	PORTA &= ~(1 << PA1);
    
    TCCR1A = 0;
    TCCR1B = 0;
    TIMSK1 = 0;
	//fast pwm icr1
    TCCR1A |= (1 << WGM11);
    TCCR1B |= (1 << WGM12) | (1 << WGM13);
	//prescaler 256
    TCCR1B |= (1 << CS12);
    TIMSK1 |= (1 << TOIE1) | (1 << OCIE1B);

    ICR1 = 0xFFFE;
	OCR1B = ICR1 / 2;

}

void init_hcsr04() {
	//set c5 as input
	DDRC &= ~(1 << PC5);
	PORTC |= (1 << PC5);
	//set c4 as output
	DDRC |= (1 << PC4);
	PORTC &= ~(1 << PC4);

	TCNT1 = 0;						
	TCCR1B |= (1<<CS10);					
	TCCR1B |= (1<<ICES1);

	PCICR = (1 << PCIE2);
	PCMSK2 = (1 << PCINT21);
}

int main()
{
	MCUCR |= (1<<JTD);
	MCUCR |= (1<<JTD);
	
 	USART0_init();
	init_engine();
	init_led();
	sei();

    _delay_ms(100);

	while(1) {
		// _delay_ms(60); 							// To allow sufficient time between queries (60ms min)
		// PORTC |= (1 << PC4);						// Set trigger high
		// _delay_us(10);							// for 10uS
		// PORTC &= ~(1 << PC4); // to trigger the ultrasonic module
	    activate_motor();
    }

    return 0;
}

